@@ -17,9 +17,9 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-/** 
+/**
  This header files defines the Signaling Abstraction Layer.
- The purpose of this layer is too allow experiment different call signaling 
+ The purpose of this layer is too allow experiment different call signaling
  protocols and implementations under linphone, for example SIP, JINGLE...
 **/
 #ifdef HAVE_CONFIG_H
@@ -39,7 +39,7 @@ const char* sal_transport_to_string(SalTransport transport) {
 		default: {
 			ms_fatal("Unexpected transport [%i]",transport);
 			return NULL;
-		}    
+		}
 	}
 }
 
@@ -62,9 +62,10 @@ SalMediaDescription *sal_media_description_new(){
 static void sal_media_description_destroy(SalMediaDescription *md){
 	int i;
 	for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;i++){
-		ms_list_for_each(md->streams[i].payloads,(void (*)(void *))payload_type_destroy);
-		ms_list_free(md->streams[i].payloads);
+		ms_list_free_with_data(md->streams[i].payloads,(void (*)(void *))payload_type_destroy);
+		ms_list_free_with_data(md->streams[i].already_assigned_payloads,(void (*)(void *))payload_type_destroy);
 		md->streams[i].payloads=NULL;
+		md->streams[i].already_assigned_payloads=NULL;
 	}
 	ms_free(md);
 }
@@ -82,28 +83,76 @@ void sal_media_description_unref(SalMediaDescription *md){
 }
 
 SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md,
-    SalMediaProto proto, SalStreamType type){
+	SalMediaProto proto, SalStreamType type){
 	int i;
-	for(i=0;i<md->n_active_streams;++i){
+	for(i=0;i<md->nb_streams;++i){
 		SalStreamDescription *ss=&md->streams[i];
+		if (!sal_stream_description_active(ss)) continue;
 		if (ss->proto==proto && ss->type==type) return ss;
 	}
 	return NULL;
 }
 
+unsigned int sal_media_description_nb_active_streams_of_type(SalMediaDescription *md, SalStreamType type) {
+	unsigned int i;
+	unsigned int nb = 0;
+	for (i = 0; i < md->nb_streams; ++i) {
+		if (!sal_stream_description_active(&md->streams[i])) continue;
+		if (md->streams[i].type == type) nb++;
+	}
+	return nb;
+}
+
+SalStreamDescription * sal_media_description_get_active_stream_of_type(SalMediaDescription *md, SalStreamType type, unsigned int idx) {
+	unsigned int i;
+	for (i = 0; i < md->nb_streams; ++i) {
+		if (!sal_stream_description_active(&md->streams[i])) continue;
+		if (md->streams[i].type == type) {
+			if (idx-- == 0) return &md->streams[i];
+		}
+	}
+	return NULL;
+}
+
+SalStreamDescription * sal_media_description_find_secure_stream_of_type(SalMediaDescription *md, SalStreamType type) {
+	SalStreamDescription *desc = sal_media_description_find_stream(md, SalProtoRtpSavpf, type);
+	if (desc == NULL) desc = sal_media_description_find_stream(md, SalProtoRtpSavp, type);
+	return desc;
+}
+
+SalStreamDescription * sal_media_description_find_best_stream(SalMediaDescription *md, SalStreamType type) {
+	SalStreamDescription *desc = sal_media_description_find_stream(md, SalProtoUdpTlsRtpSavpf, type);
+	if (desc == NULL) desc = sal_media_description_find_stream(md, SalProtoUdpTlsRtpSavp, type);
+	if (desc == NULL) desc = sal_media_description_find_stream(md, SalProtoRtpSavpf, type);
+	if (desc == NULL) desc = sal_media_description_find_stream(md, SalProtoRtpSavp, type);
+	if (desc == NULL) desc = sal_media_description_find_stream(md, SalProtoRtpAvpf, type);
+	if (desc == NULL) desc = sal_media_description_find_stream(md, SalProtoRtpAvp, type);
+	return desc;
+}
+
 bool_t sal_media_description_empty(const SalMediaDescription *md){
-	if (md->n_active_streams > 0) return FALSE;
+	if (sal_media_description_get_nb_active_streams(md) > 0) return FALSE;
 	return TRUE;
 }
 
 void sal_media_description_set_dir(SalMediaDescription *md, SalStreamDir stream_dir){
 	int i;
-	for(i=0;i<md->n_active_streams;++i){
+	for(i=0;i<md->nb_streams;++i){
 		SalStreamDescription *ss=&md->streams[i];
+		if (!sal_stream_description_active(ss)) continue;
 		ss->dir=stream_dir;
 	}
 }
 
+int sal_media_description_get_nb_active_streams(const SalMediaDescription *md) {
+	int i;
+	int nb = 0;
+	for (i = 0; i < md->nb_streams; i++) {
+		if (sal_stream_description_active(&md->streams[i])) nb++;
+	}
+	return nb;
+}
+
 
 static bool_t is_null_address(const char *addr){
 	return strcmp(addr,"0.0.0.0")==0 || strcmp(addr,"::0")==0;
@@ -114,8 +163,9 @@ static bool_t has_dir(const SalMediaDescription *md, SalStreamDir stream_dir){
 	int i;
 
 	/* we are looking for at least one stream with requested direction, inactive streams are ignored*/
-	for(i=0;i<md->n_active_streams;++i){
+	for(i=0;i<md->nb_streams;++i){
 		const SalStreamDescription *ss=&md->streams[i];
+		if (!sal_stream_description_active(ss)) continue;
 		if (ss->dir==stream_dir) return TRUE;
 		/*compatibility check for phones that only used the null address and no attributes */
 		if (ss->dir==SalStreamSendRecv && stream_dir==SalStreamSendOnly && (is_null_address(md->addr) || is_null_address(ss->rtp_addr)))
@@ -142,6 +192,52 @@ bool_t sal_media_description_has_dir(const SalMediaDescription *md, SalStreamDir
 	return FALSE;
 }
 
+bool_t sal_stream_description_active(const SalStreamDescription *sd) {
+	return (sd->rtp_port > 0);
+}
+
+bool_t sal_stream_description_has_avpf(const SalStreamDescription *sd) {
+	return ((sd->proto == SalProtoRtpAvpf) || (sd->proto == SalProtoRtpSavpf) || (sd->proto == SalProtoUdpTlsRtpSavpf));
+}
+
+bool_t sal_stream_description_has_srtp(const SalStreamDescription *sd) {
+	return ((sd->proto == SalProtoRtpSavp) || (sd->proto == SalProtoRtpSavpf));
+}
+
+bool_t sal_stream_description_has_dtls(const SalStreamDescription *sd) {
+	return ((sd->proto == SalProtoUdpTlsRtpSavp) || (sd->proto == SalProtoUdpTlsRtpSavpf));
+}
+
+bool_t sal_media_description_has_avpf(const SalMediaDescription *md) {
+	int i;
+	if (md->nb_streams == 0) return FALSE;
+	for (i = 0; i < md->nb_streams; i++) {
+		if (!sal_stream_description_active(&md->streams[i])) continue;
+		if (sal_stream_description_has_avpf(&md->streams[i]) != TRUE) return FALSE;
+	}
+	return TRUE;
+}
+
+bool_t sal_media_description_has_srtp(const SalMediaDescription *md) {
+	int i;
+	if (md->nb_streams == 0) return FALSE;
+	for (i = 0; i < md->nb_streams; i++) {
+		if (!sal_stream_description_active(&md->streams[i])) continue;
+		if (sal_stream_description_has_srtp(&md->streams[i]) != TRUE) return FALSE;
+	}
+	return TRUE;
+}
+
+bool_t sal_media_description_has_dtls(const SalMediaDescription *md) {
+	int i;
+	if (md->nb_streams == 0) return FALSE;
+	for (i = 0; i < md->nb_streams; i++) {
+		if (!sal_stream_description_active(&md->streams[i])) continue;
+		if (sal_stream_description_has_dtls(&md->streams[i]) != TRUE) return FALSE;
+	}
+	return TRUE;
+}
+
 /*
 static bool_t fmtp_equals(const char *p1, const char *p2){
 	if (p1 && p2 && strcmp(p1,p2)==0) return TRUE;
@@ -161,7 +257,7 @@ static bool_t payload_type_equals(const PayloadType *p1, const PayloadType *p2){
 	*/
 	/*
 	if (!fmtp_equals(p1->recv_fmtp,p2->recv_fmtp) ||
-	    !fmtp_equals(p1->send_fmtp,p2->send_fmtp))
+		!fmtp_equals(p1->send_fmtp,p2->send_fmtp))
 		return FALSE;
 	*/
 	return TRUE;
@@ -201,9 +297,11 @@ int sal_stream_description_equals(const SalStreamDescription *sd1, const SalStre
 	if (sd1->proto != sd2->proto) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
 	for (i = 0; i < SAL_CRYPTO_ALGO_MAX; i++) {
 		if ((sd1->crypto[i].tag != sd2->crypto[i].tag)
-			|| (sd1->crypto[i].algo != sd2->crypto[i].algo)
-			|| (strncmp(sd1->crypto[i].master_key, sd2->crypto[i].master_key, sizeof(sd1->crypto[i].master_key) - 1))) {
-			result |= SAL_MEDIA_DESCRIPTION_CRYPTO_CHANGED;
+			|| (sd1->crypto[i].algo != sd2->crypto[i].algo)){
+			result|=SAL_MEDIA_DESCRIPTION_CRYPTO_POLICY_CHANGED;
+		}
+		if ((strncmp(sd1->crypto[i].master_key, sd2->crypto[i].master_key, sizeof(sd1->crypto[i].master_key) - 1))) {
+			result |= SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED;
 		}
 	}
 
@@ -220,6 +318,10 @@ int sal_stream_description_equals(const SalStreamDescription *sd1, const SalStre
 	if (sd1->ptime != sd2->ptime) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
 	if (sd1->dir != sd2->dir) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
 
+	/*DTLS*/
+	if (sd1->dtls_role != sd2->dtls_role) result |= SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED;
+	if (strcmp(sd1->dtls_fingerprint, sd2->dtls_fingerprint) != 0) result |= SAL_MEDIA_DESCRIPTION_CRYPTO_KEYS_CHANGED;
+
 	return result;
 }
 
@@ -228,9 +330,9 @@ int sal_media_description_equals(const SalMediaDescription *md1, const SalMediaD
 	int i;
 
 	if (strcmp(md1->addr, md2->addr) != 0) result |= SAL_MEDIA_DESCRIPTION_NETWORK_CHANGED;
-	if (md1->n_total_streams != md2->n_total_streams) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
+	if (md1->nb_streams != md2->nb_streams) result |= SAL_MEDIA_DESCRIPTION_STREAMS_CHANGED;
 	if (md1->bandwidth != md2->bandwidth) result |= SAL_MEDIA_DESCRIPTION_CODEC_CHANGED;
-	for(i = 0; i < md1->n_total_streams; ++i){
+	for(i = 0; i < md1->nb_streams; ++i){
 		result |= sal_stream_description_equals(&md1->streams[i], &md2->streams[i]);
 	}
 	return result;
@@ -307,6 +409,13 @@ void sal_op_add_route_address(SalOp *op, const SalAddress *address){
 		sal_op_set_route_address(op,address);
 	}
 }
+void sal_op_set_realm(SalOp *op, const char *realm){
+	SalOpBase* op_base = (SalOpBase*)op;
+	if (op_base->realm != NULL){
+		ms_free(op_base->realm);
+	}
+	op_base->realm = ms_strdup(realm);
+}
 void sal_op_set_from(SalOp *op, const char *from){
 	SET_PARAM(op,from);
 }
@@ -372,6 +481,14 @@ const char *sal_op_get_network_origin(const SalOp *op){
 const char* sal_op_get_call_id(const SalOp *op) {
 	return  ((SalOpBase*)op)->call_id;
 }
+char* sal_op_get_dialog_id(const SalOp *op) {
+	if (op->dialog != NULL) {
+		return ms_strdup_printf("%s;to-tag=%s;from-tag=%s", ((SalOpBase*)op)->call_id,
+			belle_sip_dialog_get_remote_tag(op->dialog), belle_sip_dialog_get_local_tag(op->dialog));
+	}
+	return NULL;
+
+}
 void __sal_op_init(SalOp *b, Sal *sal){
 	memset(b,0,sizeof(SalOpBase));
 	((SalOpBase*)b)->root=sal;
@@ -401,17 +518,17 @@ void __sal_op_free(SalOp *op){
 		sal_address_destroy(b->to_address);
 		b->to_address=NULL;
 	}
-	
+
 	if (b->service_route){
 		sal_address_destroy(b->service_route);
 		b->service_route=NULL;
 	}
-	
+
 	if (b->origin_address){
 		sal_address_destroy(b->origin_address);
 		b->origin_address=NULL;
 	}
-	
+
 	if (b->from) {
 		ms_free(b->from);
 		b->from=NULL;
@@ -424,6 +541,10 @@ void __sal_op_free(SalOp *op){
 		ms_free(b->route);
 		b->route=NULL;
 	}
+	if (b->realm) {
+		ms_free(b->realm);
+		b->realm=NULL;
+	}
 	if (b->contact_address) {
 		sal_address_destroy(b->contact_address);
 	}
@@ -498,14 +619,28 @@ const char* sal_stream_type_to_string(SalStreamType type) {
 	}
 }
 
+const char *sal_stream_description_get_type_as_string(const SalStreamDescription *desc){
+	if (desc->type==SalOther) return desc->typeother;
+	else return sal_stream_type_to_string(desc->type);
+}
+
 const char* sal_media_proto_to_string(SalMediaProto type) {
 	switch (type) {
 	case SalProtoRtpAvp:return "RTP/AVP";
 	case SalProtoRtpSavp:return "RTP/SAVP";
+	case SalProtoUdpTlsRtpSavp:return "UDP/TLS/RTP/SAVP";
+	case SalProtoRtpAvpf:return "RTP/AVPF";
+	case SalProtoRtpSavpf:return "RTP/SAVPF";
+	case SalProtoUdpTlsRtpSavpf:return "UDP/TLS/RTP/SAVPF";
 	default: return "unknown";
 	}
 }
 
+const char *sal_stream_description_get_proto_as_string(const SalStreamDescription *desc){
+	if (desc->proto==SalProtoOther) return desc->proto_other;
+	else return sal_media_proto_to_string(desc->proto);
+}
+
 
 const char* sal_stream_dir_to_string(SalStreamDir type) {
 	switch (type) {
@@ -606,7 +741,7 @@ static int line_get_value(const char *input, const char *key, char *value, size_
 
 int sal_lines_get_value(const char *data, const char *key, char *value, size_t value_size){
 	int read=0;
-	
+
 	do{
 		if (line_get_value(data,key,value,value_size,&read))
 			return TRUE;
@@ -616,7 +751,7 @@ int sal_lines_get_value(const char *data, const char *key, char *value, size_t v
 }
 
 int sal_body_has_type(const SalBody *body, const char *type, const char *subtype){
-	return body->type && body->subtype 
+	return body->type && body->subtype
 		&& strcmp(body->type,type)==0
 		&& strcmp(body->subtype,subtype)==0;
 }
@@ -625,3 +760,9 @@ belle_sip_stack_t *sal_get_belle_sip_stack(Sal *sal) {
 	return sal->stack;
 }
 
+char* sal_op_get_public_uri(SalOp *op) {
+	if (op && op->refresher) {
+		return belle_sip_refresher_get_public_uri(op->refresher);
+	}
+	return NULL;
+}
