@@ -43,7 +43,8 @@ typedef struct DecState{
 	enum PixelFormat output_pix_fmt;
 	uint8_t dci[512];
 	int dci_size;
-	uint64_t last_error_reported_time;
+	MSAverageFPS fps;
+	AVFrame* orig;
 	bool_t snow_initialized;
 	bool_t first_image_decoded;
 }DecState;
@@ -52,8 +53,8 @@ typedef struct DecState{
 static void dec_init(MSFilter *f, enum CodecID cid){
 	DecState *s=(DecState *)ms_new0(DecState,1);
 	ms_ffmpeg_check_init();
-	
-	avcodec_get_context_defaults(&s->av_context);
+
+	avcodec_get_context_defaults3(&s->av_context, NULL);
 	s->av_codec=NULL;
 	s->codec=cid;
 	s->input=NULL;
@@ -69,6 +70,10 @@ static void dec_init(MSFilter *f, enum CodecID cid){
 	if (s->av_codec==NULL){
 		ms_error("Could not find decoder %i!",s->codec);
 	}
+	s->orig = av_frame_alloc();
+	if (!s->orig) {
+		ms_error("Could not allocate frame");
+	}
 
 	/*
 	s->av_context.width=MS_VIDEO_SIZE_QCIF_W;
@@ -87,13 +92,17 @@ static void dec_mpeg4_init(MSFilter *f){
 static void dec_mjpeg_init(MSFilter *f){
 	dec_init(f,CODEC_ID_MJPEG);
 }
-
 static void dec_snow_init(MSFilter *f){
+#if HAVE_ACVODEC_SNOW
 	dec_init(f,CODEC_ID_SNOW);
+#endif
 }
-
 static void dec_uninit(MSFilter *f){
 	DecState *s=(DecState*)f->data;
+	if (s->orig) {
+		av_frame_free(&s->orig);
+		s->orig = NULL;
+	}
 	if (s->av_context.codec!=NULL){
 		avcodec_close(&s->av_context);
 		s->av_context.codec=NULL;
@@ -130,18 +139,24 @@ static int dec_add_fmtp(MSFilter *f, void *data){
 static void dec_preprocess(MSFilter *f){
 	DecState *s=(DecState*)f->data;
 	int error;
-	
+
 	s->first_image_decoded = FALSE;
+	ms_average_fps_init(&s->fps, "Video decoder: FPS: %f");
 	if (s->av_context.codec==NULL){
 		/* we must know picture size before initializing snow decoder*/
+#if HAVE_AVCODEC_SNOW
 		if (s->codec!=CODEC_ID_SNOW){
-			error=avcodec_open(&s->av_context, s->av_codec);
+#endif
+			error=avcodec_open2(&s->av_context, s->av_codec, NULL);
 			if (error!=0) ms_error("avcodec_open() failed: %i",error);
 			if (s->codec==CODEC_ID_MPEG4 && s->dci_size>0){
 				s->av_context.extradata=s->dci;
 				s->av_context.extradata_size=s->dci_size;
 			}
+#if HAVE_AVCODEC_SNOW
 		}
+#endif
+
 	}
 }
 
@@ -150,13 +165,13 @@ static void dec_postprocess(MSFilter *f){
 
 static mblk_t * skip_rfc2190_header(mblk_t *inm){
 	uint8_t *ph = inm->b_rptr;
-	uint8_t sbit = (ph[0] >> 3) & 0x07; 
+	uint8_t sbit = (ph[0] >> 3) & 0x07;
 	//unsigned int ebit = ph[0] & 0x7;
 	bool_t isIFrame=0;
 	unsigned hdrLen;
 	char mode;
 	if (msgdsize(inm) < 5 ) {
-		ms_warning("RFC2190 packet too small (size %d) to scan!", msgdsize(inm));
+		ms_warning("RFC2190 packet too small (size %d) to scan!", (int)msgdsize(inm));
 		freemsg(inm);
 		return NULL;
 	}
@@ -178,7 +193,7 @@ static mblk_t * skip_rfc2190_header(mblk_t *inm){
 		inm->b_rptr += hdrLen;
 	} else {
 		ms_warning("RFC2190 packet mode:%c%s too small (size %d)", mode, isIFrame ?
-				" (I-Frame)":"", msgdsize(inm)); 
+				" (I-Frame)":"", (int)msgdsize(inm));
 		freemsg(inm);
 		inm=NULL;
 	}
@@ -192,7 +207,7 @@ static mblk_t * skip_rfc2429_header(mblk_t *inm){
 		int PLEN;
 		/*int gob_num;*/
 		bool_t P;
-		
+
 		P=rfc2429_get_P(ph);
 		PLEN=rfc2429_get_PLEN(ph);
 		/*printf("receiving new packet; P=%i; V=%i; PLEN=%i; PEBIT=%i\n",P,rfc2429_get_V(ph),PLEN,rfc2429_get_PEBIT(ph));
@@ -200,7 +215,7 @@ static mblk_t * skip_rfc2429_header(mblk_t *inm){
 		/*gob_num = (ntohl(*p) >> 10) & 0x1f;*/
 		/*ms_message("gob %i, size %i", gob_num, msgdsize(inm));
 		ms_message("ms_AVdecoder_process: received %08x %08x", ntohl(p[0]), ntohl(p[1]));*/
-		
+
 		/* remove H.263 Payload Header */
 		if (PLEN>0){
 			/* we ignore the redundant picture header and
@@ -217,7 +232,7 @@ static mblk_t * skip_rfc2429_header(mblk_t *inm){
 	}else freemsg(inm);
 	return NULL;
 }
-
+#if HAVE_AVCODEC_SNOW
 static mblk_t * parse_snow_header(DecState *s,mblk_t *inm){
 	if (msgdsize(inm) >= 4){
 		uint32_t h = ntohl(*(uint32_t*)inm->b_rptr);
@@ -225,7 +240,7 @@ static mblk_t * parse_snow_header(DecState *s,mblk_t *inm){
 			int error;
 			s->av_context.width=h>>16;
 			s->av_context.height=h&0xffff;
-			error=avcodec_open(&s->av_context, s->av_codec);
+			error=avcodec_open2(&s->av_context, s->av_codec, NULL);
 			if (error!=0) ms_error("avcodec_open() failed for snow: %i",error);
 			else {
 				s->snow_initialized=TRUE;
@@ -241,7 +256,7 @@ static mblk_t * parse_snow_header(DecState *s,mblk_t *inm){
 		return NULL;
 	}
 }
-
+#endif
 struct jpeghdr {
 	//unsigned int tspec:8;   /* type-specific field */
 	unsigned int off:32;    /* fragment byte offset */
@@ -554,7 +569,7 @@ read_rfc2435_header(DecState *s,mblk_t *inm)
 			dri = ntohs(rsthdr->dri);
 			inm->b_rptr += sizeof(struct jpeghdr_rst);
 		}
-			
+
 		if (off==0){
 			if (hdr->q>=128){
 				inm->b_rptr++; /* MBZ */
@@ -574,7 +589,7 @@ read_rfc2435_header(DecState *s,mblk_t *inm)
 				headers = allocb(495 + table_len + (dri > 0 ? 6 : 0), 0);
 				len = MakeHeaders(headers->b_rptr, hdr->type, hdr->width, hdr->height,
 					lqt_cqt, lqt_cqt+64, table_len, dri);
-				headers->b_wptr += len; 
+				headers->b_wptr += len;
 			}
 		}
 
@@ -621,7 +636,7 @@ static mblk_t *get_as_yuvmsg(MSFilter *f, DecState *s, AVFrame *orig){
 		ms_error("%s: missing rescaling context.",f->desc->name);
 		return NULL;
 	}
-#if LIBSWSCALE_VERSION_INT >= AV_VERSION_INT(0,9,0)	
+#if LIBSWSCALE_VERSION_INT >= AV_VERSION_INT(0,9,0)
 	if (sws_scale(s->sws_ctx,(const uint8_t* const*)orig->data,orig->linesize, 0,
 					ctx->height, s->outbuf.planes, s->outbuf.strides)<0){
 #else
@@ -637,22 +652,24 @@ static unsigned char smasks[7] = { 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01 };
 
 static void dec_process_frame(MSFilter *f, mblk_t *inm){
 	DecState *s=(DecState*)f->data;
-	AVFrame orig;
+
 	int got_picture;
 	/* get a picture from the input queue */
-	
+
 	if (f->desc->id==MS_H263_DEC_ID) inm=skip_rfc2429_header(inm);
 	else if (f->desc->id==MS_H263_OLD_DEC_ID) inm=skip_rfc2190_header(inm);
+#if HAVE_AVCODEC_SNOW
 	else if (s->codec==CODEC_ID_SNOW && s->input==NULL) inm=parse_snow_header(s,inm);
+#endif
 	else if (s->codec==CODEC_ID_MJPEG && f->desc->id==MS_JPEG_DEC_ID) inm=read_rfc2435_header(s,inm);
 
 	if (inm){
 		/* accumulate the video packet until we have the rtp markbit*/
 		if (s->input==NULL){
 			s->input=inm;
-		}else{ 
+		}else{
 			uint8_t sbit = (inm->reserved2 >> 11) & 0x7;
-			if (sbit!=0) {	
+			if (sbit!=0) {
 				mblk_t *mp = s->input;
 				while ( mp->b_cont != NULL ) mp = mp->b_cont;
 				mp->b_wptr--;
@@ -662,12 +679,12 @@ static void dec_process_frame(MSFilter *f, mblk_t *inm){
 			}
 			concatb(s->input,inm);
 		}
-		
+
 		if (mblk_get_marker_info(inm)){
 			mblk_t *frame;
 			int remain,len;
 			/*ms_message("got marker bit !");*/
-			/*append some padding bytes for ffmpeg to safely 
+			/*append some padding bytes for ffmpeg to safely
 			read extra bytes...*/
 			msgpullup(s->input,msgdsize(s->input)+8);
 			frame=s->input;
@@ -677,24 +694,24 @@ static void dec_process_frame(MSFilter *f, mblk_t *inm){
 				av_init_packet(&pkt);
 				pkt.data = frame->b_rptr;
 				pkt.size = remain;
-				len=avcodec_decode_video2(&s->av_context,&orig,&got_picture,&pkt);
+				len=avcodec_decode_video2(&s->av_context, s->orig, &got_picture,&pkt);
 				if (len<=0) {
 					ms_warning("ms_AVdecoder_process: error %i.",len);
-					if ((f->ticker->time - s->last_error_reported_time)>5000 || s->last_error_reported_time==0) {
-						s->last_error_reported_time=f->ticker->time;
-						ms_filter_notify_no_arg(f,MS_VIDEO_DECODER_DECODING_ERRORS);
-					}
+					ms_filter_notify_no_arg(f,MS_VIDEO_DECODER_DECODING_ERRORS);
 					break;
 				}
 				if (got_picture) {
-					mblk_t *om = get_as_yuvmsg(f,s,&orig);
+					mblk_t *om = get_as_yuvmsg(f,s,s->orig);
+					ms_average_fps_update(&s->fps, f->ticker->time);
 					if (om!=NULL)
 						ms_queue_put(f->outputs[0],om);
 
 					if (!s->first_image_decoded) {
-						ms_filter_notify_no_arg(f,MS_VIDEO_DECODER_FIRST_IMAGE_DECODED);
 						s->first_image_decoded = TRUE;
+						ms_filter_notify_no_arg(f,MS_VIDEO_DECODER_FIRST_IMAGE_DECODED);
 					}
+					if (s->codec==CODEC_ID_MJPEG)
+						break;/*assume one frame per packet for mjpeg, to workaround errors while decoding mjpeg from cameras*/
 				}
 				frame->b_rptr+=len;
 			}
@@ -716,12 +733,32 @@ static int reset_first_image(MSFilter* f, void *data) {
 	return 0;
 }
 
+static int dec_get_vsize(MSFilter *f, void *data) {
+	DecState *s = (DecState *)f->data;
+	MSVideoSize *vsize = (MSVideoSize *)data;
+	if (s->first_image_decoded == TRUE) {
+		vsize->width = s->outbuf.w;
+		vsize->height = s->outbuf.h;
+	} else {
+		vsize->width = MS_VIDEO_SIZE_UNKNOWN_W;
+		vsize->height = MS_VIDEO_SIZE_UNKNOWN_H;
+	}
+	return 0;
+}
+
+static int dec_get_fps(MSFilter *f, void *data){
+	DecState *s = (DecState *)f->data;
+	*(float*)data=ms_average_fps_get(&s->fps);
+	return 0;
+}
 
 
 static MSFilterMethod methods[]={
-	{		MS_FILTER_ADD_FMTP		,	dec_add_fmtp	},
-	{		MS_VIDEO_DECODER_RESET_FIRST_IMAGE_NOTIFICATION, reset_first_image },
-	{		0		,		NULL			}
+	{	MS_FILTER_ADD_FMTP		,	dec_add_fmtp	},
+	{	MS_VIDEO_DECODER_RESET_FIRST_IMAGE_NOTIFICATION, reset_first_image },
+	{	MS_FILTER_GET_VIDEO_SIZE,	dec_get_vsize	},
+	{	MS_FILTER_GET_FPS,		dec_get_fps	},
+	{	0		,		NULL			}
 };
 
 #ifdef _MSC_VER
@@ -806,7 +843,6 @@ MSFilterDesc ms_mjpeg_dec_desc={
 	dec_uninit,
 	methods
 };
-
 MSFilterDesc ms_snow_dec_desc={
 	MS_SNOW_DEC_ID,
 	"MSSnowDec",
@@ -822,7 +858,6 @@ MSFilterDesc ms_snow_dec_desc={
 	dec_uninit,
 	methods
 };
-
 #else
 
 MSFilterDesc ms_h263_dec_desc={
@@ -905,7 +940,6 @@ MSFilterDesc ms_mjpeg_dec_desc={
 	.uninit=dec_uninit,
 	.methods= methods
 };
-
 MSFilterDesc ms_snow_dec_desc={
 	.id=MS_SNOW_DEC_ID,
 	.name="MSSnowDec",
@@ -921,7 +955,6 @@ MSFilterDesc ms_snow_dec_desc={
 	.uninit=dec_uninit,
 	.methods= methods
 };
-
 #endif
 
 MS_FILTER_DESC_EXPORT(ms_mpeg4_dec_desc)
